@@ -19,6 +19,7 @@ import datetime as dt
 import StringIO
 import xml.etree.ElementTree as et
 import time
+import datetime as dt
 from rdfchangesets import ChangeSet, BatchChangeSet
 import json
 import re
@@ -120,10 +121,6 @@ class SparqlApi(KasabiApi):
         results.append(d)
       return response, (headers, results)
     return response, body
-
-
-class SearchApi(KasabiApi):
-  pass
   
 class LookupApi(KasabiApi):
   def lookup(self, uri, raw = False):
@@ -133,10 +130,6 @@ class LookupApi(KasabiApi):
     else:
       return response_body_as_graph(response, body)
 
-class AugmentationApi(KasabiApi):
-  pass
-class ReconciliationApi(KasabiApi):
-  pass
 class UpdateApi(KasabiApi):
   def store_data(self, data, graph_uri=None, media_type='text/turtle'):
     """Store some RDF in a graph associated with this dataset."""    
@@ -182,8 +175,109 @@ class UpdateApi(KasabiApi):
     data = g.serialize(format='xml')
     return self.client.request(self.uri, "POST", body=data, headers={"accept" : "*/*", 'content-type':'application/vnd.talis.changeset+xml', APIKEY_HEADER:self.apikey})
 
+class ReconciliationApi(KasabiApi):
+  def reconcile(self, query, limit=3, type_strict='any', type=None, properties=None, raw=False):
+    if isinstance(query, basestring):
+      # Assume this is a single label
+      param = "query=%s" % urllib.quote_plus(json.dumps(self.make_query(query, limit=limit, type_strict=type_strict, type=type, properties=properties)))
+    elif isinstance(query, list):
+      # Assume this is an array of labels
+      queries = {}
+      for i in range(0,len(query)):
+        queries['q%s'%i] = self.make_query(query[i], 3, type_strict, type, properties)
+      
+      param = "queries=%s" % urllib.quote_plus(json.dumps(queries))
+    elif isinstance(query, dict):
+      # Assume this is pre-formatted dict according to reconciliation spec
+      if 'query' in query:
+        # Assume its a single query
+        param = "query=%s" % urllib.quote_plus(json.dumps(query))
+      else:
+        # Assume its multiple queries
+        param = "queries=%s" % urllib.quote_plus(json.dumps(query))
+        
+    response, body = self.client.request("%s?%s"%(self.uri, param), "GET",headers={"accept" : "application/json", APIKEY_HEADER:self.apikey})
+    if raw or response.status not in range(200, 300):
+      return response, body
+    else:
+      data = json.loads(body)
+      return response, data
+
+  def make_query(self, label, limit=3, type_strict='any', type=None, properties=None):
+    query = {'query':label, 'limit':limit, 'type_strict':type_strict}
+    if type:
+      query['type'] = type
+    if properties:
+      query['properties'] = properties
+    
+    return query
+
+  def make_property_filter(self, value, name=None, id=None):
+    if not name and not id:
+      raise PytassiumError("Must specify at least a property name or property identifier")
+      
+    pfilter = {'v':value}
+    if name:
+      pfilter['p'] = name
+    if id:
+      pfilter['pid'] = id
+      
+    return pfilter
+
+class SearchApi(KasabiApi):
+  def search(self, query, max=None, offset=None, sort=None, raw=False):
+    params = [ "query=%s" % urllib.quote_plus(query)]
+    if max:
+      params.append("max=%s"%urllib.quote_plus(str(max)))
+    if offset:
+      params.append("offset=%s"%urllib.quote_plus(str(offset)))
+    if sort:
+      params.append("sort=%s"%urllib.quote_plus(sort))
+    
+    response, body = self.client.request("%s?%s" % (self.uri, "&".join(params)), "GET", headers={"accept" : "application/json", APIKEY_HEADER:self.apikey})
+    if raw or response.status not in range(200, 300):
+      return response, body
+    else:
+      data = json.loads(body)
+      return response, data
+
+  def facet(self, query, fields, raw=False):
+    if isinstance(fields, basestring):
+      fields = [fields]
+    response, body = self.client.request("%s/facet?query=%s&fields=%s" % (self.uri, urllib.quote_plus(query), ",".join(fields)), "GET", headers={"accept" : "application/json", APIKEY_HEADER:self.apikey})
+    if raw or response.status not in range(200, 300):
+      return response, body
+    else:
+      data = json.loads(body)
+      return response, data
+
 class JobsApi(KasabiApi):
+  def schedule_job(self, type, time = None, raw=False):
+    if time is None:
+      time = dt.datetime.utcnow()
+
+    data = { 'jobType':type, 'startTime': time.strftime('%Y-%m-%dT%H:%M:%SZ') }
+    response, body = self.client.request(self.uri, "POST",body=json.dumps(data), headers={"accept" : "*/*", 'content-type':'application/json', APIKEY_HEADER:self.apikey})
+    if raw or response.status not in range(200, 300):
+      return response, body
+    else:
+      data = response['location']
+      return response, data
+
+  def schedule_reset(self, time=None, raw=False):
+    return self.schedule_job('reset', time, raw)
+    
+  def status(self, job_uri, raw=False):
+    response, body = self.client.request(job_uri, "GET", headers={"accept" : "application/json", APIKEY_HEADER:self.apikey})
+    if raw or response.status not in range(200, 300):
+      return response, body
+    else:
+      data = json.loads(body)
+      return response, data
+
+class AugmentationApi(KasabiApi):
   pass
+
   
   
 class Dataset:
@@ -285,6 +379,37 @@ class Dataset:
       raise PytassiumError("Dataset does not have a status api")
     return api.get(raw)
 
+  def reconcile(self, query, limit=3, type_strict='any', type=None, properties=None, raw=False):
+    api = self.get_api('reconciliation')
+    if not api:
+      raise PytassiumError("Dataset does not have a reconciliation api")
+    return api.reconcile(query,raw)
+
+
+  def search(self, query, max=None, offset=None, sort=None, raw=False):
+    api = self.get_api('search')
+    if not api:
+      raise PytassiumError("Dataset does not have a search api")
+    return api.search(query,max, offset, sort, raw)
+
+  def facet(self, query, fields, raw=False):
+    api = self.get_api('search')
+    if not api:
+      raise PytassiumError("Dataset does not have a search api")
+    return api.facet(query, fields, raw)
+
+  def schedule_reset(self, time=None, raw=False):
+    api = self.get_api('jobs')
+    if not api:
+      raise PytassiumError("Dataset does not have a jobs api")
+    return api.schedule_reset(time,raw)
+
+  def job_status(self, job_uri, raw=False):
+    api = self.get_api('jobs')
+    if not api:
+      raise PytassiumError("Dataset does not have a jobs api")
+    return api.status(job_uri,raw)
+    
   def fetch_meta(self):
     response, body = self.client.request("%s.ttl"%self.uri, "GET",headers={"accept" : "text/turtle", APIKEY_HEADER:self.apikey})
     response, self.meta = response_body_as_graph(response, body, format="n3")
